/*
 * test and example for option.h
 */
#include <stdio.h>
#include <string.h>
#include <option.h>
#ifndef NO_VMT
#include <vmtio.h>
#include <vmtlib.h>
#endif

enum {
  BLACK,
  BLUE,
  GREEN,
  CYAN,
  RED,
  MAGENTA,
  BROWN,
  LIGHTGRAY,
  DARKGRAY,
  LIGHTBLUE,
  LIGHTGREEN,
  LIGHTCYAN,
  LIGHTRED,
  LIGHTMAGENTA,
  YELLOW,
  WHITE
};
char *color[] = {
  "BLACK",
  "BLUE",
  "GREEN",
  "CYAN",
  "RED",
  "MAGENTA",
  "BROWN",
  "LIGHTGRAY",
  "DARKGRAY",
  "LIGHTBLUE",
  "LIGHTGREEN",
  "LIGHTCYAN",
  "LIGHTRED",
  "LIGHTMAGENTA",
  "YELLOW",
  "WHITE",
};

#define NO    (0L)
#define LINE  (1L<<0)
#define DOT   (1L<<1)
#define POINT (1L<<2)
#define ERROR (1L<<3)
char *plot[] = {
  "NO",
  "LINE",
  "DOT",
  "POINT",
  "ERROR",
};
char *MASKS(int n, char *names[], unsigned long mask)
{
  int i = 0;
  static char masks[1024];

  if (mask == 0L) {
    strcpy(masks, names[0]);
    return masks;
  }
  strcpy(masks, "");
  for (i=1; i<n; i++)
    if (mask & 1L<<(i-1)) {
      if (strcmp(masks, "") != 0)
	strcat(masks, "|");
      strcat(masks, names[i]);
    }
  return masks;
}

char buf[256];

/*
 * global variables for examples
 */
char title[256] = "test program of option.h";
char name[256] = "option0";
char c = 'c';
int flag = !0;
int mode = !0;
int sw = 0;
int number = 0;
unsigned long mask = POINT;
int width = 1152, height = 900;
float f = 0.001;
double d = 0.002;
#ifndef NO_VMT
float *v = NULL; long nl = 1, nh;
double **m = NULL; long nrl = 1, nrh, ncl = 0, nch;
#endif

/*
 * to show variables for examples
 */
void usage(FILE *fp)
{
  fprintf(fp, "options ... \n");
  fprintf(fp, "\t-t name:    to set string (%s).   \n", title);
  fprintf(fp, "\t-n name:    to set string (%s).   \n", name);
  fprintf(fp, "\t-c char:    to set character (%c).\n", c);
  fprintf(fp, "\t-on[-]:     to set flag (%s).     \n", flag?"on":"off");
  fprintf(fp, "\t[-+]mode:   to set mode (%s).     \n", mode?"yes":"no");
  fprintf(fp, "\t[+-]sw:     to set switch (%s).   \n", sw?"on":"off");
  fprintf(fp, "\t-color enum to set enumerate (%s) \n", color[number]);
  fprintf(fp, "\t-plot mask  to set mask (%s)      \n", MASKS(5,plot,mask));
  fprintf(fp, "\t-geom WxH:  to set ints (%1dx%1d).\n", width, height);
  fprintf(fp, "\t-f float:   to set float (%f).    \n", f);
  fprintf(fp, "\t-d double:  to set double (%f).   \n", d);
#ifndef NO_VMT
  if (v) vector_sprintf(float, buf, "%g", v, nl,nh);
  fprintf(fp, "\t-v float,..: to set float vector (%s)\n", v?buf:"");
  if (m) matrix_sprintf(double, buf, "%g", m, nrl,nrh, ncl,nch);
  fprintf(fp, "\t-m (double,..),..: to set double matrix (%s)\n", m?buf:"");
#endif
  fprintf(fp, "\t-help:      show this help.       \n");
}

void main(int argc, char *argv[])
{
  int i;

  printf("*** DEFAULT SETTINGS ***\n");
  usage(stdout);

  printf("*** COMMAND LINE ***\n");
  printf("\t");
  for (i=0; i<argc; i++)
    printf("%s ", argv[i]);
  printf("\n");

  scan_option(argc, argv, "-t %s", title);
  scan_option(argc, argv, "-n \"%s\"", name);
  scan_option(argc, argv, "-c %c", &c);
  scan_option(argc, argv, "-on[-]", &flag);
  scan_option(argc, argv, "[-+]mode", &mode);
  scan_option(argc, argv, "[+-]sw", &sw);
  strcpy(buf, "-color ");
  for (i=0; i<16; i++) {
    strcat(buf, color[i]);
    strcat(buf, " ");
  }
  scan_option(argc, argv, buf, &number);
  strcpy(buf, "-plot ");
  for (i=0; i<5; i++) {
    strcat(buf, plot[i]);
    strcat(buf, "|");
  }
  scan_option(argc, argv, buf, &mask);
  scan_option(argc, argv, "-geom %dx%d", &width, &height);
  scan_option(argc, argv, "-f %f", &f);
  scan_option(argc, argv, "-d %lf", &d);
#ifndef NO_VMT
  scan_option(argc, argv, "-v %f,..", &v, &nl,&nh);
  scan_option(argc, argv, "-m (%lf,..),..", &m, &nrl,&nrh, &ncl,&nch);
#endif
  if (scan_option(argc, argv, "-help")) {
    printf("*** SETTINGS BY YOUR OPTIONS ***\n");
    usage(stdout);
    fprintf(stdout, "\tThere is help option.\n");
    exit(0);
  }

  printf("*** SETTINGS BY YOUR OPTIONS ***\n");
  usage(stdout);

  exit(0);
}
