/*
 * Routines to analyze command-line options < options.c >
 *
 * ޥɥ饤󥪥ץϤΤΥ롼
 *
 * Copyright (C) 1996,1997,1999 Taiji Yamada
 */
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <options.h>
#ifndef NO_VMT
#include "vmtio.h"
#include "vmtlib.h"
#endif

#define n_options __n_options
#define options __options
static int n_options = 0;
static option_complex *options = NULL;

option_complex *set_option(char *key, ...)
{
  int i, n;
  char opt[1024], *ptr;
  option_complex *it;
  va_list argptr;

  n = n_options;
  n_options++;
  if (options == NULL)
    options = (option_complex *)malloc(n_options*sizeof(option_complex));
  else
    options = (option_complex *)realloc(options,
					n_options*sizeof(option_complex));
  if (options == NULL) return NULL;
  it = &options[n];
  it->fmt = NULL;
  it->c = 0;
  it->var = NULL;
  it->idx = 0;

  strcpy(opt, key);
  /*
   * scanf style option and its variations
   */
  if ((ptr=strchr(opt,'%')) != NULL) {
#ifndef NO_VMT    
    char *p;

    if ((p=strstr(opt,",..")) != NULL) {
      char *q, *r;

      /*
       * matrix option such as "-m (%lf,..),..",
       * use like "-m (1,2,3),(4,5,6),(7,8,9)".
       */
      if (((q=strstr(opt,"("))!=NULL)&&((r=strstr(p+1,",.."))!=NULL)) {
	it->key = (char *)malloc((strlen(opt)+1)*sizeof(char));
	strcpy(it->key, opt);
	it->fmt = (char *)malloc((p-ptr+1)*sizeof(char));
	strncpy(it->fmt, ptr, p-ptr);
	it->fmt[p-ptr] = '\0';
	it->c = 5;
	it->var = (void **)malloc(it->c*sizeof(void *));
	va_start(argptr, key);
	it->var[0] = (void *)va_arg(argptr,void ***);
	it->var[1] = (void *)va_arg(argptr,long *);
	it->var[2] = (void *)va_arg(argptr,long *);
	it->var[3] = (void *)va_arg(argptr,long *);
	it->var[4] = (void *)va_arg(argptr,long *);
	va_end(argptr);
      }
      /*
       * vector option such as "-v %lf,..", use like "-v 1,2,3".
       */
      else {
	it->key = (char *)malloc((strlen(opt)+1)*sizeof(char));
	strcpy(it->key, opt);
	it->fmt = (char *)malloc((p-ptr+1)*sizeof(char));
	strncpy(it->fmt, ptr, p-ptr);
	it->fmt[p-ptr] = '\0';
	it->c = 3;
	it->var = (void **)malloc(it->c*sizeof(void *));
	va_start(argptr, key);
	it->var[0] = (void *)va_arg(argptr,void **);
	it->var[1] = (void *)va_arg(argptr,long *);
	it->var[2] = (void *)va_arg(argptr,long *);
	va_end(argptr);
      }
    }
#endif
    if (strstr(opt,"\"%s\"") != NULL) ptr = strstr(opt,"\"%s\"");
    it->fmt = (char *)malloc((strlen(ptr)+1)*sizeof(char));
    strcpy(it->fmt, ptr);
    *ptr = '\0';
    while (*(--ptr)==' ') *ptr = '\0';
    it->key = (char *)malloc((strlen(opt)+1)*sizeof(char));
    strcpy(it->key, opt);
    ptr = strchr(it->fmt, '%');
    while (ptr != NULL) {
      it->c++;
      ptr = strchr(ptr+1, '%');
    }
    it->var = (void **)malloc(it->c*sizeof(void *));
    va_start(argptr, key);
    for (i=0; i<it->c; i++)
      it->var[i] = va_arg(argptr, void *);
    va_end(argptr);
  }
  /*
   * flag type option such as "[-+]on" or "[+-]on", use like "-on" or "+on".
   * "[-+]" style gives "-on": !0 and "+on": 0, and
   * "[+-]" style gives "+on": !0 and "-on": 0.
   */
  else if ((ptr=strstr(opt,"[-+]")) != NULL||
           (ptr=strstr(opt,"[+-]")) != NULL) {
    it->key = (char *)malloc((strlen(opt)+1)*sizeof(char));
    strcpy(it->key, opt);
    va_start(argptr, key);
    it->c = 1;
    it->var = (void **)malloc(it->c*sizeof(void *));
    it->var[0] = va_arg(argptr, int *);
    va_end(argptr);
  }
  /*
   * flag type option such as "-on[-]", use like "-on": !0 or "-on-": 0.
   */
  else if ((ptr=strstr(opt,"[-]")) != NULL) {
    it->key = (char *)malloc((strlen(opt)+1)*sizeof(char));
    strcpy(it->key, opt);
    va_start(argptr, key);
    it->c = 1;
    it->var = (void **)malloc(it->c*sizeof(void *));
    it->var[0] = va_arg(argptr, int *);
    va_end(argptr);
  }
  /*
   * bit-mask type option such as "-plot NO|LINE|DOT|POINT",
   * use like "-plot LINE" or "-plot DOT" or "-plot 'LINE|POINT'".
   */
  else if ((ptr=strstr(opt," ")) != NULL && strstr(ptr,"|") != NULL) {
    it->key = (char *)malloc((strlen(opt)+1)*sizeof(char));
    strcpy(it->key, opt);
    va_start(argptr, key);
    it->c = 1;
    it->var = (void **)malloc(it->c*sizeof(void *));
    it->var[0] = va_arg(argptr, unsigned long *);
    va_end(argptr);
  }
  /*
   * enumerate type option such as "-color RED GREEN BLUE",
   * use like "-color RED" or "-color GREEN" or "-color BLUE".
   */
  else if ((ptr=strstr(opt," ")) != NULL) {
    it->key = (char *)malloc((strlen(opt)+1)*sizeof(char));
    strcpy(it->key, opt);
    va_start(argptr, key);
    it->c = 1;
    it->var = (void **)malloc(it->c*sizeof(void *));
    it->var[0] = va_arg(argptr, int *);
    va_end(argptr);
  }
  /*
   * another style options such as "-help",
   * use like "-help" and 'if (is_option("-help")){...}' in C.
   */
  else {
    it->key = (char *)malloc((strlen(opt)+1)*sizeof(char));
    strcpy(it->key, opt);
  }
  return it;
}

option_complex *alias_option(char *key, char *alias)
{
#define VLIST10(V) V[0],V[1],V[2],V[3],V[4],V[5],V[6],V[7],V[8],V[9]
  int n;
  option_complex *it;

  for (n=0; n<n_options; n++) {
    char buf[1024];

    it = &options[n];
    if (strcmp(key, it->key) == 0) {
      strcpy(buf, alias);
      if (it->fmt) {
        strcat(buf, " ");
        strcat(buf, it->fmt);
      }
      return set_option(buf, VLIST10(it->var));
    }
  }
  return NULL;
#undef VLIST10
}

void set_options(int n, option_record *opts)
{
  int i;
  option_complex *it;

  for (i=0; i<n; i++)
    it = set_option(opts[i].opt, opts[i].var);
}

void scan_options(int argc, char *argv[])
{
#define VLIST10(V) V[0],V[1],V[2],V[3],V[4],V[5],V[6],V[7],V[8],V[9]
  int i, n;
  char opt[1024], *ptr;
  option_complex *it;

  for (n=0; n<n_options; n++) {
#ifndef NO_VMT
    char *p, buf[1024];

#endif
    it = &options[n];
    strcpy(opt, it->key);
    /*
     * scanf style option and its variations
     */
#ifndef NO_VMT
    if ((p=strstr(opt,",..")) != NULL) {
      char *q, *r, *s = ptr;

      /*
       * matrix option such as "-m (%lf,..),..",
       * use like "-m (1,2,3),(4,5,6),(7,8,9)".
       */
      if (((q=strstr(opt,"("))!=NULL)&&((r=strstr(p+1,",.."))!=NULL)) {
        long *nrl, *nrh, *ncl, *nch;
	void ***m = NULL;
	fcctype_t type;
	size_t size;
	
	/*while (*(--q) == ',') (*nrl)++;
	while (*(--s) == ',') (*ncl)++;*/ /* rejected idea */
        while (*(--q) == ' '); *(++q) = '\0';
        for (i=1; i<argc-1; i++)
          if (strcmp(argv[i], opt)==0) {
	    /*if (m && *m) {
	      (matrix_free)(size, *m, *nrl, *ncl);
	      m = NULL;
	    }*/ /* critical idea */
            it->idx = i;
            strcpy(buf, argv[it->idx+1]);
	    type = fccode_type(it->fmt);
	    size = fccode_size(it->fmt);
	    (matrix_sscanf)(size, buf, it->fmt, (m=(void ***)it->var[0]), *(nrl=(long *)it->var[1]), (nrh=(long *)it->var[2]), *(ncl=(long *)it->var[3]), (nch=(long *)it->var[4]));
          }
	continue;
      }
      /*
       * vector option such as "-v %lf,..", use like "-v 1,2,3".
       */
      else {
        long *nl, *nh;
        void **v = NULL;
	fcctype_t type;
	size_t size;

	/*while (*(--s) == ',') (*ncl)++;*/ /* rejected idea */
        while (*(--s) == ' '); *(++s) = '\0';
        for (i=1; i<argc-1; i++)
          if (strcmp(argv[i], opt)==0) {
	    /*if (v && *v) {
	      (vector_free)(size, *v, *nl);
	      v = NULL;
	    }*/ /* critical idea */
            it->idx = i;
            strcpy(buf, argv[it->idx+1]);
	    type = fccode_type(it->fmt);
	    size = fccode_size(it->fmt);
	    (vector_sscanf)(size, buf, it->fmt, (v=(void **)it->var[0]), *(nl=(long *)it->var[1]), (nh=(long *)it->var[2]));
          }
	continue;
      }
    }
#endif
    if (it->fmt) {
      char buf[1024];

      for (i=1; i<argc; i++)
        if (strcmp(argv[i], it->key)==0) {
          it->idx = i;
          break;
        }
      if (it->idx != 0) {
        /*
         * double quoted string option such as '-interp \"%s\"',
         * use like '-interp \"ghostview -\"'
         */
        if (strcmp(it->fmt,"\"%s\"") == 0) {
          if (it->idx+1 < argc && argv[it->idx+1][0] == '\"') {
            strcpy(buf, "");
            strcat(buf, argv[++i]+1);
            strcat(buf, " ");
            while (++i < argc) {
              strcat(buf, argv[i]);
              if ((ptr=strchr(buf,'\"')) != NULL) {
                *ptr = '\0';
                break;
              }
              if (i != argc-1) strcat(buf, " ");
            }
            strcpy(it->var[0], buf);
          } else
            strcpy(it->var[0], argv[it->idx+1]);
        }
        /*
         * option including some spaces such as "-title %s",
         * use like '-title "Title of this program"'.
         */
        else if (strcmp(it->fmt,"%s") == 0) {
          if (it->idx+1 < argc) strcpy(it->var[0], argv[it->idx+1]);
        }
        /*
         * normal scanf style options such "-geometry %dx%d",
         * use like "-geometry 640x480".
         */
        else {
          strcpy(buf, "");
          i = it->idx;
          while (++i < argc) {
            strcat(buf, argv[i]);
            strcat(buf, " ");
          }
          sscanf(buf, it->fmt, VLIST10(it->var));
        }
      }
    }
    /*
     * flag type option such as "[-+]on" or "[+-]on", use like "-on" or "+on".
     * "[-+]" style gives "-on": !0 and "+on": 0, and
     * "[+-]" style gives "+on": !0 and "-on": 0.
     */
    else if ((ptr=strstr(opt,"[-+]")) != NULL||
             (ptr=strstr(opt,"[+-]")) != NULL) {
      char buf[1024];
      
      strcpy(buf, (*(ptr+1)=='-')?"-":"+");
      strcat(buf, opt+strlen("[-+]"));
      for (i=1; i<argc; i++) {
        if (strcmp(argv[i], buf)==0) {
          *(int *)it->var[0] = !0;
          it->idx = i;
          /* ! */
        }
      }
      strcpy(buf, (*(ptr+1)=='-')?"+":"-");
      strcat(buf, opt+strlen("[-+]"));
      for (i=1; i<argc; i++) {
        if (strcmp(argv[i], buf)==0) {
          *(int *)it->var[0] = 0;
          it->idx = i;
          /* ! */
        }
      }
    }
    /*
     * flag type option such as "-on[-]", use like "-on": !0 or "-on-": 0.
     */
    else if ((ptr=strstr(opt,"[-]")) != NULL) {
      char buf[1024];

      *ptr = '\0';
      strcpy(buf, opt);
      for (i=1; i<argc; i++) {
        if (strcmp(argv[i], buf)==0) {
          *(int *)it->var[0] = !0;
          it->idx = i;
          /* ! */
        }
      }
      strcat(buf, "-");
      for (i=1; i<argc; i++) {
        if (strcmp(argv[i], buf)==0) {
          *(int *)it->var[0] = 0;
          it->idx = i;
          /* ! */
        }
      }
    }
    /*
     * bit-mask type option such as "-plot NO|LINE|DOT|POINT",
     * use like "-plot LINE" or "-plot DOT" or "-plot 'LINE|POINT'".
     */
    else if ((ptr=strstr(opt," ")) != NULL && strstr(ptr,"|") != NULL) {
      unsigned long b, *p = (unsigned long *)(it->var[0]);

      ptr = strtok(opt, " |");
      for (i=1; i<argc; i++)
        if (strcmp(argv[i], opt)==0) {
	  int flag = 0;

	  b = 0L;
          ptr = strtok(NULL, " |");
          while (ptr && i+1<argc) {
	    char *s = argv[i+1];

	    do {
	      if (strncmp(s, ptr, strlen(ptr))==0) {
		*p = flag?(*p|b):b;
		flag = !0;
	      }
	      s = strstr(s, "|");
	      s = s?(s+1):NULL;
	    } while (s != NULL);
	    b = (b == 0L)?1L:(b<<1);
            ptr = strtok(NULL, " |");
          }
	  if (flag)
	    it->idx = i;
        }
    }
    /*
     * enumerate type option such as "-color RED GREEN BLUE",
     * use like "-color RED" or "-color GREEN" or "-color BLUE".
     */
    else if ((ptr=strstr(opt," ")) != NULL) {
      int c;
      
      ptr = strtok(opt, " ");
      for (i=1; i<argc; i++)
        if (strcmp(argv[i], opt)==0) {
          ptr = strtok(NULL, " ");
          c = 0;
          while (ptr && i+1<argc) {
            if (strcmp(argv[i+1], ptr)==0) {
              *(int *)(it->var[0]) = c;
              it->idx = i;
            }
            c++;
            ptr = strtok(NULL, " ");
          }
        }
    }
    /*
     * another style options such as "-help",
     * use like "-help" and 'if (scan_option(argc,argv,"-help")){...}' in C.
     */
    else {
      for (i=1; i<argc; i++) {
        if (strcmp(argv[i], it->key)==0) {
          it->idx = i;
          break;
        }
      }
    }
  }
#undef VLIST10
}

int is_option(char *key)
{
  int n;
  option_complex *it;

  for (n=0; n<n_options; n++) {
    it = &options[n];
    if (strcmp(key, it->key) == 0) { /* ! */
      return it->idx;
    }
  }
  return 0;
}

void reset_options(void)
{
  int n;
  option_complex *it;

  for (n=0; n<n_options; n++) {
    it = &options[n];
    if (it->c != 0)
      free(it->var);
    if (it->fmt != NULL) free(it->fmt);
    if (it->key != NULL) free(it->key);
  }
  free(options);
  options = NULL;
  n_options = 0;
}
