/*
 * test for vmtio.h
 *
 * Copyright (C) 1999 Taiji Yamada <taiji@aihara.co.jp>
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "vmtio.h"
#include "vmtlib.h"

#define v_l 0
#define v_h 19

#define m_rl 1
#define m_rh 5
#define m_cl 0
#define m_ch 9

#define t_rl 0
#define t_rh 2
#define t_cl 1
#define t_ch 3
#define t_dl -1
#define t_dh 1

long V_l = v_l;
long V_h = v_h;
long M_rl = m_rl;
long M_rh = m_rh;
long M_cl = m_cl;
long M_ch = m_ch;
long W_rl = m_rl;
long W_rh = m_rh;
long W_cl = m_cl;
long W_ch = m_ch;
long T_rl = t_rl;
long T_rh = t_rh;
long T_cl = t_cl;
long T_ch = t_ch;
long T_dl = t_dl;
long T_dh = t_dh;

void main()
{
  int i, j, k;
  char buf[1024];
  {
    int *v, **m, ***t;
    int *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(int, v_l,v_h);
    m = matrix_alloc(int, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(int, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(int, buf, "%03d", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(int, buf, "%03d", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(int, "%03d", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(int, buf, "%03d", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(int, buf, "%03d", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(int, "%03d", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(int, buf, "%03d", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(int, buf, "%03d", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(int, "%03d", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(int, buf, "%03d", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(int, buf, "%03d", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(int, "%03d", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(int, v, v_l,v_h);
    vector_free(int, V, V_l,V_h);
    matrix_free(int, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(int, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(int, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(int, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(int, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
  {
    short *v, **m, ***t;
    short *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(short, v_l,v_h);
    m = matrix_alloc(short, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(short, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(short, buf, "%03hd", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(short, buf, "%03hd", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(short, "%03hd", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(short, buf, "%03hd", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(short, buf, "%03hd", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(short, "%03hd", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(short, buf, "%03hd", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(short, buf, "%03hd", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(short, "%03hd", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(short, buf, "%03hd", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(short, buf, "%03hd", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(short, "%03hd", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(short, v, v_l,v_h);
    vector_free(short, V, V_l,V_h);
    matrix_free(short, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(short, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(short, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(short, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(short, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
  {
    long *v, **m, ***t;
    long *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(long, v_l,v_h);
    m = matrix_alloc(long, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(long, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(long, buf, "%03ld", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(long, buf, "%03ld", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(long, "%03ld", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(long, buf, "%03ld", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(long, buf, "%03ld", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(long, "%03ld", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(long, buf, "%03ld", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(long, buf, "%03ld", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(long, "%03ld", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(long, buf, "%03ld", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(long, buf, "%03ld", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(long, "%03ld", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(long, v, v_l,v_h);
    vector_free(long, V, V_l,V_h);
    matrix_free(long, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(long, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(long, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(long, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(long, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
  {
    unsigned int *v, **m, ***t;
    unsigned int *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(unsigned int, v_l,v_h);
    m = matrix_alloc(unsigned int, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(unsigned int, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(unsigned int, buf, "%03u", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(unsigned int, buf, "%03u", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(unsigned int, "%03u", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(unsigned int, buf, "%03u", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(unsigned int, buf, "%03u", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(unsigned int, "%03u", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(unsigned int, buf, "%03u", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(unsigned int, buf, "%03u", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(unsigned int, "%03u", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(unsigned int, buf, "%03u", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(unsigned int, buf, "%03u", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(unsigned int, "%03u", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(unsigned int, v, v_l,v_h);
    vector_free(unsigned int, V, V_l,V_h);
    matrix_free(unsigned int, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(unsigned int, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(unsigned int, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(unsigned int, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(unsigned int, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
  {
    unsigned short *v, **m, ***t;
    unsigned short *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(unsigned short, v_l,v_h);
    m = matrix_alloc(unsigned short, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(unsigned short, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(unsigned short, buf, "%03hu", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(unsigned short, buf, "%03hu", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(unsigned short, "%03hu", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(unsigned short, buf, "%03hu", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(unsigned short, buf, "%03hu", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(unsigned short, "%03hu", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(unsigned short, buf, "%03hu", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(unsigned short, buf, "%03hu", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(unsigned short, "%03hu", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(unsigned short, buf, "%03hu", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(unsigned short, buf, "%03hu", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(unsigned short, "%03hu", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(unsigned short, v, v_l,v_h);
    vector_free(unsigned short, V, V_l,V_h);
    matrix_free(unsigned short, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(unsigned short, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(unsigned short, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(unsigned short, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(unsigned short, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
  {
    unsigned long *v, **m, ***t;
    unsigned long *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(unsigned long, v_l,v_h);
    m = matrix_alloc(unsigned long, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(unsigned long, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(unsigned long, buf, "%03lu", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(unsigned long, buf, "%03lu", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(unsigned long, "%03lu", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(unsigned long, buf, "%03lu", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(unsigned long, buf, "%03lu", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(unsigned long, "%03lu", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(unsigned long, buf, "%03lu", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(unsigned long, buf, "%03lu", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(unsigned long, "%03lu", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(unsigned long, buf, "%03lu", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(unsigned long, buf, "%03lu", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(unsigned long, "%03lu", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(unsigned long, v, v_l,v_h);
    vector_free(unsigned long, V, V_l,V_h);
    matrix_free(unsigned long, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(unsigned long, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(unsigned long, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(unsigned long, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(unsigned long, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
  {
    float *v, **m, ***t;
    float *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(float, v_l,v_h);
    m = matrix_alloc(float, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(float, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(float, buf, "%03g", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(float, buf, "%03g", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(float, "%03g", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(float, buf, "%03g", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(float, buf, "%03g", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(float, "%03g", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(float, buf, "%03g", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(float, buf, "%03g", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(float, "%03g", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(float, buf, "%03g", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(float, buf, "%03g", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(float, "%03g", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(float, v, v_l,v_h);
    vector_free(float, V, V_l,V_h);
    matrix_free(float, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(float, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(float, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(float, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(float, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
  {
    double *v, **m, ***t;
    double *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(double, v_l,v_h);
    m = matrix_alloc(double, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(double, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(double, buf, "%03g", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(double, buf, "%03lg", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(double, "%03g", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(double, buf, "%03g", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(double, buf, "%03lg", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(double, "%03g", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(double, buf, "%03g", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(double, buf, "%03lg", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(double, "%03g", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(double, buf, "%03g", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(double, buf, "%03lg", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(double, "%03g", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(double, v, v_l,v_h);
    vector_free(double, V, V_l,V_h);
    matrix_free(double, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(double, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(double, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(double, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(double, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
  {
    long double *v, **m, ***t;
    long double *V = NULL, **M = NULL, **W = NULL, ***T = NULL;

    v = vector_alloc(long double, v_l,v_h);
    m = matrix_alloc(long double, m_rl,m_rh, m_cl,m_ch);
    t = f3tensor_alloc(long double, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);

    for (i=v_l; i<=v_h; i++)
      v[i] = i-v_l;
    for (i=m_rl; i<=m_rh; i++)
      for (j=m_cl; j<=m_ch; j++)
	m[i][j] = (i-m_rl)*(m_ch-m_cl+1)+(j-m_cl);
    for (i=t_rl; i<=t_rh; i++)
      for (j=t_cl; j<=t_ch; j++)
	for (k=t_dl; k<=t_dh; k++)
	  t[i][j][k] = (i-t_rl)*(t_dh-t_dl+1)*(t_ch-t_cl+1)+(j-t_cl)*(t_dh-t_dl+1)+(k-t_dl);

    vector_sprintf(long double, buf, "%03Lg", v, v_l,v_h);
    printf("# v: %s\n", buf);
    vector_sscanf(long double, buf, "%03Lg", &V, V_l,&V_h);
    printf("# V:\n");
    vector_printf(long double, "%03Lg", V, V_l,V_h);
    printf("\n");
    matrix_sprintf(long double, buf, "%03Lg", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m: %s\n", buf);
    matrix_sscanf(long double, buf, "%03Lg", &M, M_rl,&M_rh, M_cl,&M_ch);
    printf("# M:\n");
    matrix_printf(long double, "%03Lg", M, M_rl,M_rh, M_cl,M_ch);
    printf("\n");
    matrixT_sprintf(long double, buf, "%03Lg", m, m_rl,m_rh, m_cl,m_ch);
    printf("# m^T: %s\n", buf);
    matrixT_sscanf(long double, buf, "%03Lg", &W, W_rl,&W_rh, W_cl,&W_ch);
    printf("# W:\n");
    matrix_printf(long double, "%03Lg", W, W_rl,W_rh, W_cl,W_ch);
    printf("\n");
    f3tensor_sprintf(long double, buf, "%03Lg", t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    printf("# t: %s\n", buf);
    f3tensor_sscanf(long double, buf, "%03Lg", &T, T_rl,&T_rh, T_cl,&T_ch, T_dl,&T_dh);
    printf("# T:\n");
    f3tensor_printf(long double, "%03Lg", T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
    printf("\n");

    vector_free(long double, v, v_l,v_h);
    vector_free(long double, V, V_l,V_h);
    matrix_free(long double, m, m_rl,m_rh, m_cl,m_ch);
    matrix_free(long double, M, M_rl,M_rh, M_cl,M_ch);
    matrix_free(long double, W, W_rl,W_rh, W_cl,W_ch);
    f3tensor_free(long double, t, t_rl,t_rh, t_cl,t_ch, t_dl,t_dh);
    f3tensor_free(long double, T, T_rl,T_rh, T_cl,T_ch, T_dl,T_dh);
  }
}
