/*
 * memory allocator for any type of vector, matrix, tensor < vmtlib.c >
 *
 * VMT: Vector Matrix Tensor lib, Numerial Recipes in C compatible
 *
 * Copyright (C) 1999 Taiji Yamada
 */
#include <stdlib.h>

#define NR_COMPATIBLE 1

void *(vector_alloc)(size_t size, long nl, long nh)
{
  void *v = malloc(size*(nh-nl+1+NR_COMPATIBLE));
  if (!v)
    return NULL;
  else
    return v - size*(nl-NR_COMPATIBLE);
}

void (vector_free)(size_t size, void *v, long nl)
{
  free(v + size*(nl-NR_COMPATIBLE));
}

void **(matrix_alloc)(size_t size, long nrl, long nrh, long ncl, long nch)
{
  long i, nr=nrh-nrl+1, nc=nch-ncl+1;
  void **m, **tmp;

  tmp = m = malloc(sizeof(void *)*(nr+NR_COMPATIBLE));
  if (!m)
    return NULL;
  m -= (nrl-NR_COMPATIBLE);
  *(m+nrl) = malloc(size*(nr*nc+NR_COMPATIBLE));
  if (!*(m+nrl)) {
    free(tmp);
    return NULL;
  }
  *(m+nrl) -= size*(ncl-NR_COMPATIBLE);
  for (i=nrl+1; i<=nrh; i++)
    *(m+i) = *(m+i-1) + size*nc;
  return m;
}

void (matrix_free)(size_t size, void **m, long nrl, long ncl)
{
  free(*(m+nrl) + size*(ncl-NR_COMPATIBLE));
  free(m + (nrl-NR_COMPATIBLE));
}

void ***(f3tensor_alloc)(size_t size, long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
{
  long i, j, nr=nrh-nrl+1, nc=nch-ncl+1, nd=ndh-ndl+1;
  void ***t, ***tmp, **temp;

  tmp = t = (void ***)malloc(sizeof(void **)*(nr+NR_COMPATIBLE));
  if (!t)
    return NULL;
  t -= (nrl-NR_COMPATIBLE);
  temp = *(t+nrl) = (void **)malloc(sizeof(void *)*(nr*nc+NR_COMPATIBLE));
  if (!*(t+nrl)) {
    free(tmp);
    return NULL;
  }
  *(t+nrl) -= (ncl-NR_COMPATIBLE);
  *(*(t+nrl)+ncl) = (void *)malloc(size*(nr*nc*nd+NR_COMPATIBLE));
  if (!*(*(t+nrl)+ncl)) {
    free(temp);
    free(tmp);
    return NULL;
  }
  *(*(t+nrl)+ncl) -= size*(ndl-NR_COMPATIBLE);
  for (j=ncl+1; j<=nch; j++)
    *(*(t+nrl)+j) = *(*(t+nrl)+j-1) + size*nd;
  for (i=nrl+1; i<=nrh; i++) {
    *(t+i) = *(t+i-1) + nc;
    *(*(t+i)+ncl) = *(*(t+i-1)+ncl) + size*nc*nd;
    for (j = ncl + 1; j <= nch; j++)
      *(*(t+i)+j) = *(*(t+i)+j-1) + size*nd;
  }
  return t;
}

void (f3tensor_free)(size_t size, void ***t, long nrl, long ncl, long ndl)
{
  free(*(*(t+nrl)+ncl) + size*(ndl-NR_COMPATIBLE));
  free(*(t+nrl) + (ncl-NR_COMPATIBLE));
  free(t + (nrl-NR_COMPATIBLE));
}
